/* **********************************************************************************************************
 *	Das Modul beinhaltet Event Listener fr die Behandlung der Keyboard Eingaben
 *	in die Datums- und Uhrzeit Eingabefelder, Daylight Savings Time, Zeitzone, Lngen-
 *	und Breitengrad.
 *
 *	Jedes Objekt hat 3 Eventlistener fr "keydown", "keyup" und "change". Dazu gehren 
 *	dann bei "keydown" und "change" jeweils eine Callback Funktion und bei "keydown" das
 *	setzen einer Variablen. SIEHE DIE BESCHREIBUNG DER LISTENER. Die Funktionen
 *	sind in Keyboard.js definiert.
 *
 *
 *	The module includes event listener for handling keyboard input
 *	in the date and time input fields, Daylight Savings Time, Time Zone, Longitude
 *	and latitude.
 *
 *	Each object has 3 event listers for keydown, keyup and change.  These include: 
 *	then with keydown and change each a callback function and with keydown the
 *	set a variable.  SEE  DESCRIPTION  THE LISTENER.The functions
 *	are defined in Keyboard.js.
 *
 *
 *	Events der Objekte:
 *	---------------------------
 *
 *	-	objB {body}										-	Globalen Tastendruck im Body abfangen (Button Berechnen,
 *																	Bild Mondphase)
 *																	To intercept global keystroke in the body (Calculate button,
 *																	picture moon phase)
 *	-	objDay{Eingabe Tag}						-	Tastatureingabe des Tages
 *																	Keyboard input of the day
 *	-	objMonth{Eingabe Monat}					-	Tastatureingabe des Monats
 *																	Keyboard input of the month
 *	-	objYear{Eingabe Tag}						-	Tastatureingabe des Jahres
 *																	Keyboard input of the year
 *	-	objDST{Eingabe Sommer/Winter}		-	Tastatureingabe des Tages
 *																	Keyboard input of the day
 *	-	objTZ{Eingabe Zeitzone}					-	Tastatureingabe der Zeitzone
 *																	Keyboard input of the time zone
 *	-	objLongitude{Eingabe Lngengrad}	-	Tastatureingabe des Lngengrades
 *																	Keyboard input of the longitude
 *	-	objLatitude{Eingabe Breitengrad}		-	Tastatureingabe des Breitengrades
 *																	Keyboard input of the latitude
 *
 *
 *	25.04.2021, Dieter Steffan
 *	25.05.2021, Dieter Steffan						Update Beschreibungen
 *
 *********************************************************************************** ****************************/

// Tastaturevent im Body global abfangen um Buttons bedienen zu knnen.
// Intercept keyboard event in the body globally to be able to operate buttons.
objB.addEventListener("keydown",  function(e){
	// Tastendruck "b" - Press "b"
	if(e.which == 66){
		// Die Callback Funktion des in Keyboard.js definierten Listener aufrufen.
		// Call the callback function of the listener defined in Keyboard. js
		// Simuliert den Klick auf den "Berechnen" Button
		// Simulates the click on the "Calculate" button
		bodyBerechnenClick(e, ".calc");
	}// Tastendruck "p" - Press "p"
	else if(e.which == 80){
		bodyMoonPositionClick(e, ".phasepicture");
	}// Tastendruck "Enter" - Press "Enter"
	else if(e.which == 13){
		// Startet das Programm - starts the program
		bodyBerechnenClick(e, ".calc");
	}
	// Fokus auf das Hauptprogramm.
	objB.focus();
});
					
					
/* ********************************************************************************** */

					
// Objekt Tag mit Event "keydown".
// Object tag with event "keydown".
objDay.addEventListener("keydown",  function (e) {
	// Funktion Eingabefeld Tag Keys erlauben/verbieten Callback  in Keyboard.js
	// Function Input field Tag Keys allow/disallow callback  in Keyboard.js
	keydownInputDay(e);
});
	
// Objekt Tag mit Event "change".
// Test when changing the text field.
objDay.addEventListener("change", function(){
	// Funktion Eingabefeld Tag verlassen und prfen Callback in Keyboard.js
	// Function Input field Leave tag and check callback in Keyboard.js
	keydownChangeDay(objDay, objMonth, objLeapYear);
});
					
// Der Listener wird aufgerufen, wenn bei der Tageseingabe
// eine Taste losgelassen wird ...
// The listener is called up when a key is released during the day entry ...
objDay.addEventListener("keyup", function (e){
	// ... und wenn es eine der Sondertasten Shift, Strg, Alt ist,
	// dann setze die Variable fr die verbotenen Sondertasten zurck.
	// . . . and if it is one of the special keys Shift, Ctrl, Alt,
	// then reset the variable for the forbidden special keys.
	if (e.which >= SHIFTLEFT && e.which <= ALTLEFT){
		deniedKeyDay = false;
	}
});


/* ********************************************************************************** */

					
// Monat Tastaturanschlge Erlauben/Verbieten und erster.
// Month Keystrokes Allow/Forbid and first.
objMonth.addEventListener("keydown",  function (e) {
	// Eingabefeld Monat Keys erlauben/verbieten Callback in Keyboard.js
	// Input field Month Keys Allow/Ban Callback in Keyboard.js
	keydownInputMonth(e);
});
	
// Objekt Monat mit Event "change".
// Object month when changing the text field.
objMonth.addEventListener("change", function(){
	// Funktion Eingabefeld Monat verlassen und prfen Callback in Keyboard.js
	// Function Input field Leave month and check callback in Keyboard.js
	keydownChangeMonth(objMonth);
});
					
// Der Listener wird aufgerufen, wenn bei der Tageseingabe
// eine Taste losgelassen wird ...
// The listener is called up when a key is released during the day entry ...
objMonth.addEventListener("keyup", function (e){
	// ... und wenn es eine der Sondertasten Shift, Strg, Alt ist,
	// dann setze die Variable fr die verbotenen Sondertasten zurck.
	// . . . and if it is one of the special keys Shift, Ctrl, Alt,
	// then reset the variable for the forbidden special keys.
	if (e.which >= SHIFTLEFT && e.which <= ALTLEFT){
		deniedKeyDay = false;
	}
});


/* ********************************************************************************** */

					
// Objekt Jahr Tastaturanschlge Erlauben/Verbieten und erster.
// Object year Keystrokes Allow/Forbid and first.
objYear.addEventListener("keydown",  function (e) {
	// Funktion Eingabefeld Jahr Keys erlauben/verbieten Callback in Keyboard.js
	// Function input field year Keys Allow/Ban Callback in Keyboard.js
	keydownInputYear(e);
});
	
// Objekt Jahr mit Event "change".
// Object year when changing the text field.
objYear.addEventListener("change", function(){
	// Funktion Eingabefeld Jahr verlassen und prfen Callback in Keyboard.js
	// Function Input field Leave year and check callback in Keyboard.js
	keydownChangeYear(objYear, objLeapYear);
});
				
// Der Listener wird aufgerufen, wenn bei der Jahreseingabe
// eine Taste losgelassen wird ...
// The listener is called up when a key is released during the year entry ...
objYear.addEventListener("keyup", function (e){
	// ... und wenn es eine der Sondertasten Shift, Strg, Alt ist,
	// dann setze die Variable fr die verbotenen Sondertasten zurck.
	// . . . and if it is one of the special keys Shift, Ctrl, Alt,
	// then reset the variable for the forbidden special keys.
	if (e.which >= SHIFTLEFT && e.which <= ALTLEFT){
		deniedKeyYear = false;
	}
});


/* ********************************************************************************** */

								
// Objekt Lngengrad Tastaturanschlge Erlauben/Verbieten und erster.
// Object longitude Keystrokes Allow/Forbid and first.
objLongStep.addEventListener("keydown", function (e){
	// Funktion Eingabefeld Lngengrad Keys erlauben/verbieten Callback in Keyboard.js
	// Function input field longitude Keys Allow/Ban Callback in Keyboard.js
	keydownInputLongStep(e);
});

// Objekt Lngengrad mit Event "change".
// Object longituder when changing the text field.
objLongStep.addEventListener("change", function(){
	// Funktion Eingabefeld Lngengrad verlassen und prfen Callback in Keyboard.js
	// Function Input field Leave longitude and check callback in Keyboard.js
	keydownChangeLongStep(objLongStep, objLatStep);
});

// Das Textfeld fr die Eingabe der Lngengrad Schrittweite wird verlassen
//  The text field for entering the longitude step width is left
objLongStep.addEventListener("focusout", function(){
	// Funktion Eingabefeld Lngengrad Fokus und prfen Callback in Keyboard.js
	// Function Input field focus longitude and check callback in Keyboard.js
	keydownChangeLongStep(objLongStep, objLatStep);
	FIRSTCALC = false;
});
			
// Der Listener wird aufgerufen, wenn bei der Eingabe der Lngengrad Schrittweite
// eine Taste losgelassen wird ...
// The listener is called up when a key is released during the longitude step values entry ...
objLongStep.addEventListener("keyup", function (e){
	// ... und wenn es eine der Sondertasten Shift, Strg, Alt ist,
	// dann setze die Variable fr die verbotenen Sondertasten zurck.
	// . . . and if it is one of the special keys Shift, Ctrl, Alt,
	// then reset the variable for the forbidden special keys.
	if (e.which >= SHIFTLEFT && e.which <= ALTLEFT){
		deniedKeyLongStep = false;
	}
});

				
/* ********************************************************************************** */


// Objekt Breitengrad Schrittweite der Berechnung Tastaturanschlge Erlauben/Verbieten
// Object latitude step values Keystrokes Allow/Forbid
objLatStep.addEventListener("keydown", function (e) {
	// Funktion Eingabefeld breitengrad Keys erlauben/verbieten Callback in Keyboard.js
	// Function input field latitude Keys Allow/Ban Callback in Keyboard.js
	keydownInputLatStep(e);
});

// Objekt Breitengrad mit Event "change".
// Object latituder when changing the text field.
objLatStep.addEventListener("change", function() {
	// Funktion Eingabefeld Breitengrad verlassen und prfen Callback in Keyboard.js
	// Function Input field Leave latitude and check callback in Keyboard.js
	keydownChangeLatStep(objLatStep, objLongStep);
});

// Das Textfeld fr die Eingabe der Breitengrad Fokus
//  The text field for entering the latitude the focus
objLatStep.addEventListener("focusout", function() {
	// Funktion Eingabefeld Breitengrad Fokus und prfen Callback in Keyboard.js
	// Function Input field jocus latitude and check callback in Keyboard.js
	keydownChangeLatStep(objLatStep, objLongStep);
});
					
// Der Listener wird aufgerufen, wenn bei der Eingabe der zeitzone
// eine Taste losgelassen wird ...
// The listener is called up when a key is released during the timezone entry ...
objLatStep.addEventListener("keyup", function (e) {
	// ... und wenn es eine der Sondertasten Shift, Strg, Alt ist,
	// dann setze die Variable fr die verbotenen Sondertasten zurck.
	// . . . and if it is one of the special keys Shift, Ctrl, Alt,
	// then reset the variable for the forbidden special keys.
	if (e.which >= SHIFTLEFT && e.which <= ALTLEFT){
		deniedKeyLatStep = false;
	}
});

