/* **********************************************************************************************************
 *	Das Modul beinhaltet Callback Funktionen fr die Behandlung der Keyboard Eingaben
 *	in die Datums- und Uhrzeit Eingabefelder, Daylight Savings Time, Zeitzone, Lngen-
 *	und Breitengrad. Die Tastatur EventListener sind in Keyboard-Listener.js definiert.
 *
 *	The module includes callback functions for handling keyboard inputs
 *	in the date and time input fields, Daylight Savings Time, Time Zone, Longitude
 *	and latitude.  The EventListener keyboard is defined in Keyboard-Listener.js.
 *
 *
 *	Funktionsliste:
 * --------------------
 *
 *
 *	- keydownInputDay(e)									- Eingabefeld Tag Keys erlauben/verbieten Callback
 *	- keydownChangeDay(objDay)						- Eingabefeld Tag verlassen und prfen Callback
 *	- keydownInputMonth(e)								- Eingabefeld Monat Keys erlauben/verbieten Callback
 *	- keydownChangeMonth(objMonth)				- Eingabefeld Monat verlassen und prfen Callback
 * -  keydownInputYear(e)									- Eingabefeld Jahr Keys erlauben/verbieten Callback
 * -  keydownChangeYear(objYear)					- Eingabefeld Jahr verlassen und prfen Callback
 * -  keydownInputTZ(e)									- Eingabefeld Timezone Keys erlauben/verbieten Callback
 * -  keydownChangeTZ(objTZ)							- Eingabefeld Timezone verlassen und prfen Callback
 * -  keydownInputDST(e)									- Eingabefeld Sommer/Winterzeit Keys erlauben/verbieten Callback
 * -  keydownChangeDST(objDST)					- Eingabefeld Sommer/Winterzeit verlassen und prfen Callback
 * -  bodyBerechnenClick(e, classButton)			- Simuliert einen Klick auf den Button "Berechnen" wenn Taste "b" gedrckt wird
 * -  bodyMoonPositionClick(e, classButton)		- Simuliert einen Klick auf ddas MoonPhase Bild wenn Taste "p" gedrckt wird
 *
 *	17.10.2020	Dieter Steffan
 *	20.05.2021	Dieter Steffan							Update Beschreibungen
 *	02.06.2021	Dieter Steffan							Funktionen fr lokale Version entfernt
 *************************************************************************************************************** */

 
// Konstanten erlaubte/verbotene Tastenanschlge
// which 			= 8 			= Backspace
// 						= 46 		= Delete (Entf)
// 						= 37 		= Pfeil Links
// 						= 39 		= Pfeil Rechts
// 						= 48 - 57 	= Zahlen 0 - 9
// 						= 16 		= Linke Umschalttaste (Shift)Taste
// 						= 17			= Linke Steuerungs (Control)Taste
// 						= 18 		= Linke Alt Taste
//						= 189		= Minus Zeichen
//						= 16 + 189= Unterstrich
//						= 190		= Punkt
//						= 221 		= Rckstrich
// Constants allowed/prohibited keystrokes
// which 			= 8 			= Backspace
// 						= 46 		= Delete
// 						= 37 		= Left Arrow
// 						= 39 		= Right Arrow
// 						= 48 - 57 	= Numbers 0 - 9
// 						= 16 		= Left Shift Key
// 						= 17 		= Left Control Key
// 						= 18 		= Left Alt Key
// 						= 189 		= Minus Sign
//						= 16 + 189= Underscore
// 						= 190 		= Dot
//						= 221 		= Backstroke
const BACKSPACE = 8;
const DELETE = 46;
const ARROWLEFT = 37;
const ARROWRIGHT = 39;
const BELOWNUMBER = 48;
const ONENUMBER = 49;
const ABOVENUMBER = 57;
const TAB = 9;
const POS1 = 36;
const END =  35;
const SHIFTLEFT =  16;
const CONTROLLEFT = 17;
const ALTLEFT =  18;
const MINUS = 189;
const POINT = 190;
const BACKSTROKE = 221;
const UNDERSCORE = 16;
const UNDERSCORE1 = 189;

// Bei der Taste "b" wird ein Klick auf den Button "Berechnen"
// simuliert und der Button bedient und somit die Berechnung
// gestartet.
// At the key "b" click on the button "Calculate"
// simulated and the button operated and thus the calculation
// started.
function bodyBerechnenClick(e, classButton){
		$(classButton).click();
		
} // End of bodyBerechnenClick()



// Bei der Taste "p" wird ein Klick auf das Bild der Mondphase
// simuliert und das Popup Fenter geffnet sowie die Berechnung
// gestartet.
// With the key "p" a click on the image of the moon phase
// simulated and opened the popup finder as well as the calculation
// started.
function bodyMoonPositionClick(e, classButton){
		$(classButton).click();
		
} // End of bodyMoonPositionClick()




 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDay.addEventListener("keydown",keydownInputDay);
		
	fr die Tastaturanschlge des Einagbefeldes fr den Tag,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objDay. addEventListener("keydown",keydownInputDay); 
			
	for the keystrokes of the input field for the day, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyDay = false;
function keydownInputDay(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyDay = true;
		// Hier werden die nicht erlaubten Tastendrcke und die Erlaubten abgewiesen und zugelassen.
		// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
		// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
		// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
		// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
		// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
		if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyDay == false))||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
					(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END)) ){
					; // ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
		}else{
				// ... ist die Funktion vorhanden? ... dann fhre sie aus ...
				// ... is the function present?  ... then perform them ...
				if (e.preventDefault()) {
					// Cancel the event
					// If you prevent default behavior in Mozilla Firefox prompt will always be shown
					e.stopPropagation().preventDefault(); 
				// ... ansonsten Returnwert setzen und Standardaktion zulassen ...
				//  ... otherwise set return value and allow standard action ...
				}else{
					// Chrome requires returnValue to be set
					e.returnValue = false;
					return (e.returnValue);
				}
		}
} // End of keydownInputDay()




 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDay.addEventListener("change", function(){
						keydownChangeDay(objDay, objMonth, objLeapYear);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl dem eines Tages im Monat
	entspricht. Es wird auch das Schaltjahr geprft, allerdings sollte
	vor Berechnungsbeginn noch eimal ein Test erfolgen, da hier ja die
	Reihenfolge der Eingaben nicht festgehalten ist. Somit kann es zu
	Fehlern kommen.
	
	bergabewert:
	objDay					= Das Objekt des zu testenden Textfelds
	objMonth				= Das Objekt desTextfelds fr den Monat
	objLeapYear			= Das Objekt des Textfelds fr das Schaltjahr
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objDay. addEventListener("change", function(){ 
			keydownChangeDay(objDay, objMonth, objLeapYear);
		});
		
	for testing for plausible entries when leaving the text field. Here, 
	we only check whether the number corresponds to one day in the month. 
	The leap year is also checked, but before the calculation starts a test 
	should be done once more, because the order of the entries is not fixed 
	here. This can lead to errors.
	
	Transfer value:
	objDay 				= The object of the text field to be tested
	objMonth 			= The object of the text field for the month
	objLeapYear 	= The object of the text field for the leap year
	
	Return value:
	None
  *************************************************************************************** */
  // Array mit den erlaubten Tagen des Monats im Jahr. Februarr ist der
  //  Wert fr das normale Jahr.
  // Array with the allowed days of the month in the year. Februaryr is the
  // value for the normal year.
 var allowedDays = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
function  keydownChangeDay(objDay, objMonth, objLeapYear){
	// Variable fr die erlaubten Tage nach dem Monat definieren und initialisieren.
	// Define and initialize variable for the allowed days after the month.
	var alloweddays = 0;
	// Kein Schaltjahr und Monat ist Februar ...
	// No leap year and month is February . . .
	if ((objLeapYear.value == "Nein") && (parseInt(objMonth.value) == 2))
		// ... dann hole den entsprechenden Wert aus dem Array. Monat ist minus 1 ...
		// . . . then get the corresponding value from the array. Month is minus 1 . . .
		alloweddays = allowedDays[parseInt(objMonth.value) -  1];
	// Haben wir ein Schaltjahr und der Monat ist Februar ...
	// If we have a leap year and the month is February. . .
	else if ((objLeapYear.value == "Ja") && (parseInt(objMonth.value )== 2))
		// ... dann sind 29 Tage erlaubt ...
		// . . . then 29 days are allowed . . .
		alloweddays= 29;
	// ... fr alle anderen Flle hole wieder die erlaubten Tage, nach dem Monat
	// aus dem Array ...
	// . . . for all other cases get back the allowed days, after the month
	// from the array . . .
	else{
		alloweddays = allowedDays[parseInt(objMonth.value) - 1];
	}
	// Tag hat keinen Wert ...
	// Day has no value. . .
	if (objDay.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nEs muss ein Wert zwischen 1 und 31 eingetragen werden!");	
		objDay.value = 1;
		objDay.focus();
	// ... oder der Wert ist 0 ...
	// . . . or the value is 0 . . .
	}else if (parseInt(objDay.value)== 0){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr einen Tag nicht erlaubt!\nEs muss ein Wert zwischen 1 und 31 eingetragen werden!");
		objDay.value = 1;		
		objDay.focus();
	// ... oder der Wert ist grer als 31 ...
	// . . . or the value is greater than 31 . . .
	}else if (parseInt(objDay.value)> alloweddays){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Den Tag " +  objDay.value + " gibt es in einem Monat nicht!\nEs muss ein Wert zwischen 1 und " +  alloweddays + " eingetragen werden!");	
		objDay.value = alloweddays;
		objDay.focus();
	}
	
} // End of keydownChangeDay()



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMonth.addEventListener("keydown",  function (e) {
			keydownInputMonth(e);
		});	
		
	fr die Tastaturanschlge des Eingabefeldes fr den Monat,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objMonth.addEventListener("keydown",  function (e) {
			keydownInputMonth(e);
		});	
			
	for the keystrokes of the input field for the month, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyMonth = false;
function keydownInputMonth(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyMonth = true;
		// Hier werden die nicht erlaubten Tastendrcke und die Erlaubten abgewiesen und zugelassen.
		// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
		// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
		// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
		// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
		// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
		if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyMonth == false))||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
					(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
					; // ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
		}else{
				// ... ist die Funktion vorhanden? ... dann fhre sie aus ...
				// ... is the function present?  ... then perform them ...
				if (e.preventDefault()) {
					// Cancel the event
					// If you prevent default behavior in Mozilla Firefox prompt will always be shown
					e.stopPropagation().preventDefault(); 
				// ... ansonsten Returnwert setzen und Standardaktion zulassen ...
				//  ... otherwise set return value and allow standard action ...
				}else{
					// Chrome requires returnValue to be set
					e.returnValue = false;
					return (e.returnValue);
				}
		}	
} // End of keydownInputMonth()



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMonth.addEventListener("change", function(){
						keydownChangeMonth(objMonth);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl dem eines Monats im Jahr
	entspricht. Der Monat Februar bei Schaltjahr ja/nein wird hier noch nicht
	bercksichtigt und muss am Ende der Eingabe bzw. bei Berechnungsstart
	erfolgen.
	
	bergabewert:
	objMonth			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objMonth. addEventListener("change", function(){ 
			keydownChangeMonth(objMonth); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one month in the year.
	The month February with leap year yes/no is not yet considered here
	and must be entered at the end of the input or at the start of the calculation.
	
	Transfer value:
	objMonth = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeMonth(objMonth){
	// Tag hat keinen Wert ...
	// Day has no value. . .
	if (objMonth.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr den Monat muss ein Wert zwischen 1 und 12 eingetragen werden!");	
		objMonth.value = 1;
		objMonth.focus();
	// ... oder der Wert ist 0 ...
	// . . . or the value is 0 . . .
	}else if (parseInt(objMonth.value)== 0){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr einen Monat nicht erlaubt!\nEs muss ein Wert zwischen 1 und 12 eingetragen werden!");	
		objMonth.value = 1;
		objMonth.focus();
	// ... oder der Wert ist grer als 31 ...
	// . . . or the value is greater than 31 . . .
	}else if (parseInt(objMonth.value)>  12){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Den Tag " +  objMonth.value + " gibt es in einem Monat nicht!\nEs muss ein Wert zwischen 1 und 12 eingetragen werden!");
		objMonth.value = 12;
		objMonth.focus();
	}

} // End of keydownChangeMonth()


/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objYear.addEventListener("keydown",keydownInputMonth);
		
	fr die Tastaturanschlge des Eingabefeldes fr das Jahr,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objYear. addEventListener("keydown",keydownInputYear); 
			
	for the keystrokes of the input field for the year, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
 // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyYear = false;
function keydownInputYear(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyYear = true;
		// Hier werden die nicht erlaubten Tastendrcke und die Erlaubten abgewiesen und zugelassen.
		// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
		// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
		// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
		// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
		// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
		if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyYear== false))||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
					(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
					; // ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
					// Zeigt an, das das Jahr noch nicht berechnet wurde
					// Shows that the year has not yet been calculated
					yearIsCalculated = false;
		}else{
				// ... ist die Funktion vorhanden? ... dann fhre sie aus ...
				// ... is the function present?  ... then perform them ...
				if (e.preventDefault()) {
					// Cancel the event
					// If you prevent default behavior in Mozilla Firefox prompt will always be shown
					e.stopPropagation().preventDefault(); 
				// ... ansonsten Returnwert setzen und Standardaktion zulassen ...
				//  ... otherwise set return value and allow standard action ...
				}else{
					// Chrome requires returnValue to be set
					e.returnValue = false;
					return (e.returnValue);
				}
		}	
} // End of keydownInputYear()



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objYear.addEventListener("change", function(){
						keydownChangeYear(objYear);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl dem eines Jahr es
	entspricht. Der Monat Februar bei Schaltjahr ja/nein wird hier noch nicht
	bercksichtigt und muss am Ende der Eingabe bzw. bei Berechnungsstart
	erfolgen.
	
	bergabewert:
	objMonth			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objYear. addEventListener("change", function(){ 
			keydownChangeYear(objYear); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the year.
	The month February with leap year yes/no is not yet considered here
	and must be entered at the end of the input or at the start of the calculation.
	
	Transfer value:
	objYear = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeYear(objYear, objLeapYear){
	// Tag hat keinen Wert ...
	// Day has no value. . .
	if (objYear.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr das Jahr muss ein Wert > 1599 eingetragen werden!");	
		objYear.value = 2000;
		objYear.focus();
	// ... oder der Wert ist 0 ...
	// . . . or the value is 0 . . .
	}else if (parseInt(objYear.value)== 0){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr das Jahr nicht erlaubt!\nEs muss ein Wert > 1599 eingetragen werden!");	
		objYear.value = 2000;
		objYear.focus();
	// ... oder der Wert ist kleiner 1600 ...
	// . . . or the value is less than 1600 . . .
	}else if (parseInt(objYear.value)< 1600){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr das Jahr nicht erlaubt!\nEs muss ein Wert > 1599 eingetragen werden!");	
		objYear.value = 1600;
		objYear.focus();
	}

	// Ist das Berechnungsjahr ein Schaltjahr?
	// Is the calculation year a leap year?
		if (isLeapYear(objYear.value))
			objLeapYear.value = "Ja";
		else
		objLeapYear.value = "Nein";

} // End of keydownChangeYear()



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
			objLongStep.addEventListener("keydown", function (e){
				keydownInputLongStep(e);
			});	
		
	fr die Tastaturanschlge des Eingabefeldes fr die Schritteweite der 
	Berechnungen der Lngengrade. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	--------------------
		e	=	Event des objLongStep.addEventListener("keydown", function (e) ...
	
	Returnwert:
	---------------
		Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event list:
	
 			objLongStep.addEventListener (keydown , function (e)  {
 				keydownInputLongStep  (e)  ;
 			}) ;	
		
 	for the keyboard stops of the input field for the step width of the 
 	Calculations of longitude.  Here are the keystrokes
 	filtered according to the permission.
	
 	Transfer value:
 	--------------------
 		e	 =	 Event of the objLongStep.addEventListener (keydown, function (e)...
	
 	Return value:
 	---------------
 		None
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyLongStep = false;
function keydownInputLongStep(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if ( (e.which >= SHIFTLEFT && e.which <= ALTLEFT) )deniedKeyLongStep = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ( (e.which >= BELOWNUMBER) && (e.which <= ABOVENUMBER) && (deniedKeyLongStep == false) ) ||
					( ( (e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT) ||
					(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1) || (e.which ==  END) || (e.which ==  POINT) ) && (deniedKeyLongStep == false) )  ){
				// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... ist die Funktion vorhanden? ... dann fhre sie aus ...
				// ... is the function present?  ... then perform them ...
				if (e.preventDefault()) {
					// Cancel the event
					// If you prevent default behavior in Mozilla Firefox prompt will always be shown
					e.stopPropagation().preventDefault(); 
				// ... ansonsten Returnwert setzen und Standardaktion zulassen ...
				//  ... otherwise set return value and allow standard action ...
				}else{
					// Chrome requires returnValue to be set
					e.returnValue = false;
					return (e.returnValue);
				}
			}
			
} // End of keydownInputLongStep()



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objLongStep.addEventListener("change", function(){
						keydownChangeLongStep(objLongStep);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird geprft, ob die Lngengrad Schrittweite der Iteration ber
	die Oberflche der Erde ein teilbarer Wert von 360 ist. 
	
	bergabewert:
	--------------------
		objLongStep	= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	----------------
		Keiner
	---------------------------------------------------------------------------------------------------
	Die Funktion ist das Callback des Eventlisteners:
	
		objLongStep.addEventListener("change", function(){
						keydownChangeLongStep(objLongStep);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl der Schrittweite des Lngengrades
	dem Wert 1 - 100.0 entspricht. 
	
	bergabewert:
	--------------------
		objLongStep	= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	----------------
		Keiner
  *************************************************************************************** */
var SAVELONGSTEP = 0.0;	// Sichert den letzten Lngengrad Wert
											// Saves the last longitude value
var FIRSTCALC = false;
function  keydownChangeLongStep(objLongStep, objLatStep){
	var LATSTEP = Number(objLatStep.value);
	var LONGSTEP = Number(objLongStep.value);
	var result = new Array();
	// Wenn sich 360 durch die Schrittweite ohne Rest teilen lsst und
	// im erlaubten Bereich liegt ...
	//  If 360 can be divided by the step distance without remainder and
 	//  within the permitted area ...
	if(  ((360.0 % LONGSTEP) == 0) && (LONGSTEP <= 40) ){
		;	// ... dann tue nichts und akzeptiere die Eingabe ...
			//  ... then do nothing and accept the input ...
	// ... im unerlaubten Bereich ...
	// ... in the unauthorized area ...
	}else{
		// Lngengrad Schrittweite hat keinen Wert ...
		// Longitude step value has no value. . .
		if (objLongStep.value == ""){
			// ... dann eine Meldung ausgeben ...
			// . . . then display a message. . .
			alert("Das Eingabefeld ist leer!\nFr die Lngengrad Schrittweite muss ein Wert eingetragen werden!");
			// ... und trage den doppelten Wert des Lngengrades in das Feld des Lngengrades ein ...
			// ... and enter twice the value of the longitude in the field of the longitude ...
			objLongStep.value = LATSTEP * 2;
			objLongStep.focus();
		// ... der Lngengrad liegt ber dem erlaubten Wert von 40 ...
		// . . . the longitude is above the allowed value of 40 ...
		}else if (parseInt(objLongStep.value) >  40){
			// ... dann eine Meldung ausgeben ...
			// . . . then display a message. . .
			alert("Der Wert grer 40 ist fr die Lngengrad Schrittweite nicht erlaubt!\nEs muss ein Wert 1- 40 eingetragen werden!");	
			// ... und trage den Maximalert des Lngengrades in das Feld des Lngengrades ein ...
			// ... and enter twice the value of the longitude in the field of the longitude ...
			objLongStep.value = 40;
			objLongStep.focus();
		// ... die restlichen Flle der unerlaubten Werte ...
		//  ... the remaining cases of illicit values ...
		}else{
			// ... und zuerst wenn er grer 0 ist ...
			// ...  and first when it is greater than 0 ...
			if (objLongStep.value != 0.0){
				// ... prfe ob der gesicherte Wert von vorher deselbe ist wie der aktuelle und wenn, ...
				//  ... check if the backed-up value from before is the same as the current value and if, ...
				if ( SAVELONGSTEP ==  Number(objLongStep.value) ) {
					// ... dann vergingere den Lngengrad um Nachkommaanteil ...
					//  ... then decrease the longitude by  Descendant share ...
					if(FIRSTCALC == false){
						FIRSTCALC = true;
						// ... 360 durch den eingegebenen Lngengrad ohne Rest teilen ...
						//  ... 360  Diving by the entered longitude without remainder ...
						result = getDecimalPoints(objLongStep.value);
						// ... und dann den nchsten ganzzahligen Divisor ermitteln ...
						// ...  and then determine the next integer divisor ...
						LONGSTEP = getNearestDivisor(360.0, result[0], result[1], result[2]);
				}else
						LONGSTEP = LONGSTEP - 1 ;
				}
				// ... schreibe Ihn in das Eingabetextfeld ...
				// ... write it in the input text field ...
				objLongStep.value = LONGSTEP;
				// ... und sichere Ihn auch noch ...
				// ... and secure him also ...
				SAVELONGSTEP = objLongStep.value;
				// ... und rufe mich dann rekursiv fr einen weiteren Test auf ...
				// ...  and then call me recursively for another test ...
				keydownChangeLongStep(objLongStep, objLatStep);
			// ... und hier ist der Wert 0 ...
			// ...  and here is the value 0 ...
			}else if (objLongStep.value == 0.0){
					// ... dann eine Meldung ausgeben ...
					// . . . then display a message. . .
					alert("Das Eingabefeld hat den Wert 0!\nFr die Lngengrad Schrittweite muss ein Wert eingetragen werden!");
					// ... und trage den zweifachen Breitengrad Wert in das Feld des Lngengrades ein ...
					// ... and enter twice the value of the latitude in the field of the longitude ...
					objLongStep.value = LATSTEP * 2;
					objLongStep.focus();
			}
		}
	}
	// Zum Abschluss kontrolliere die Bretiengradangabe.
	//  Finally, check the latitude grade indication.
	keydownChangeLatStep(objLatStep, objLongStep);
	
} // End of keydownChangeLongStep()



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDST.addEventListener("keydown",keydownInputDST);
		
	fr die Tastaturanschlge des Eingabefeldes fr die Zeitzone,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objTZ. addEventListener("keydown",keydownInputTZ); 
			
	for the keystrokes of the input field for the timezone, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyLatStep = false;
function keydownInputLatStep(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyLatStep = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ( (e.which >= BELOWNUMBER) && (e.which <= ABOVENUMBER) && (deniedKeyLatStep == false) ) ||
					( ( (e.which ==  BACKSPACE) || (e.which ==  DELETE) || (e.which ==  ARROWLEFT) || 
						(e.which ==  ARROWRIGHT) || (e.which ==  TAB) || (e.which ==  POS1) || (e.which ==  END) || (e.which ==  POINT) ) && (deniedKeyLatStep == false) )  ){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... ist die Funktion vorhanden? ... dann fhre sie aus ...
				// ... is the function present?  ... then perform them ...
				if (e.preventDefault()) {
					// Cancel the event
					// If you prevent default behavior in Mozilla Firefox prompt will always be shown
					e.stopPropagation().preventDefault(); 
				// ... ansonsten Returnwert setzen und Standardaktion zulassen ...
				//  ... otherwise set return value and allow standard action ...
				}else{
					// Chrome requires returnValue to be set
					e.returnValue = false;
					return (e.returnValue);
				}
			}
			
} // End of keydownInputLatStep()




/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objTZ.addEventListener("change", function(){
						keydownChangeTZ(objTZ);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl der Zeitzone von -12 - +12
	entspricht. 
	
	bergabewert:
	objTZ			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objTZ.addEventListener("change", function(){ 
			keydownChangeDST(objTZ); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the timezone
	-12 - +12.
	
	Transfer value:
	objTZ = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
var SAVELATSTEP = 0.0;
function  keydownChangeLatStep(objLatStep, objLongStep){
	var LATSTEP = Number(objLatStep.value);
	var LONGSTEP = Number(objLongStep.value);
	// Der Breitengrad muss ohne Rest teilbar sein und kleiner gleich 20 ...
	// The latitude must be divisible without remainder and less equal to 20 ...
	if( ( (180.0 % LATSTEP) == 0) && (LATSTEP <= 20) ){
		// ... Der Wert ist nicht die Hlfte der Lngengrad Schrittweite ...
		// . . . or the value is not the half of the longitude step value ...
		if ( (LATSTEP >  (LONGSTEP / 2.0) ) || (LATSTEP <  (LONGSTEP / 2.0) ) ){
			// ... dann eine Meldung ausgeben ...
			// . . . then display a message. . .
			objLatStep.value = LONGSTEP / 2.0;
			objLatStep.focus();
		}
	}else{
		// Breitengrad Schrittweite hat keinen Wert ...
		// Latitude step value has no value. . .
		if (LATSTEP == ""){
			// ... dann eine Meldung ausgeben ...
			// . . . then display a message. . .
			objLatStep.value = LONGSTEP / 2.0;
			objLatStep.focus();
		}else if (LATSTEP >  20){
			// ... dann eine Meldung ausgeben ...
			// . . . then display a message. . .
			objLatStep.value = LONGSTEP / 2.0;
			objLatStep.focus();
		// ... in allen anderen Fllen, wenn der Breitengrad noch nicht stimmt ...
		// ... in all other cases, if the latitude is not yet correct ...
		}else{
			// ... wenn der gesicherte Breitengrad gleich dem Eingabefeld ist ...
			// ... if the secured latitude is equal to the input field ...
			if ( SAVELATSTEP ==  Number(objLatStep.value) ) {
				// ... und der Lngengrad gleich 0, dann ...
				// ... and the longitude equals 0, then ...
				if(SAVELONGSTEP == 0.0)
					// ... veringere den Breitengrad um den Betrag des Nachkommaanteils ...
				// ... reduce the latitude by the amount of the decimal part ...
					LATSTEP = LATSTEP - frac(Number(objLatStep.value));
				// ... bei allen anderen Lngengraden ...
				// ... for all other degrees of longitude ...
				else
					// ... veringere den Breitengrad um den Betrag 1
					// ... reduce the latitude by the amount 1
					LATSTEP = LATSTEP - 1;
			}
			// ... Breitengrad Schritt Ausgeben ...
			// ... Latitude Step Output ...
			objLatStep.value = LATSTEP;
			// ... dann sichern ...
			// ... then save ...
			SAVELATSTEP = objLatStep.value;
			// ... und Funktion rekursiv aufrufen, bis der richtige Wert gefunden ist, also
			// den Wert des Breitengrades, der ohne Rest durch den Lngengrad teilbar ist.
			// ... and call function recursively until the correct value is found, i.e.
			// the value of the latitude, which is divisible by the longitude without remainder.
			keydownChangeLatStep(objLatStep, objLongStep);
		}
	}
}// End of keydownChangeLatStep()


