/*	**********************************************************************************************************
	Das Modul enthält Funktionen und Ereignis Callbacks, die für
	die Benutzung eines eigenen modalen Dialog Fensters nötig sind.
	Es simuliert quasi das alert() Kommando.
	
	Es sind Funktionen vorhanden, wie:
	----------------------------------------------
	
	resetVars()						-	Reset die Variablen in Global-this.js
	stopBackgroundWorker()	-	Stopt den Background  Worker, der in Background_worker.js
												definiert ist
	startBackgroundWorker()	-	Startet den Background  Worker, der in Background_worker.js
												definiert ist
	goAndWork()					-	Startet die Verarbeitung (startBackgroundWorker())und sendet Daten zum 
												Background Worker und an das Ausgabefenster
	Rückruf Funktion resize	-	Für die Postion des Dialogfensters in der Mitte des Browsers
	Rückruf Funktion click		-	Für die Buttons "OK" und "Abbrechen"
	Rückruf Funktion keydown-	Für die ESC-Taste, um die Bearbeitung abzubrechen und Dialogfenster
												zu schliessen
	Rückruf Funktion focus		-	Für den Fokus auf dem Dialogfenster
	showDialog()					-	Zeigt das Dialogfenster an oder verbirgt es	

	
	Hierfür sind folgende HTML-Elemente notwendig:
	------------------------------------------------------------------
	
	<!-- Bereich <div> zwischen Body und Dialog mit der Abdunkelung -->
	<div id="trans" style="visibility: hidden; position:absolute; width: 0px; height:0px; left: 0px; top: 0px; z-index: 1;"></div>
	<!-- Dialog Box mit Textfeld und Buttons -->
	<div tabindex="-1" style="position: absolute; top: 200px; left:200px; width:300px; height:200px;" role="dialog" aria-lablledby="mydialog" id="dialog" class="box-hidden">
		<div id="txt" role="dialog" aria-lablledby="mydialog" style="color: black; text-align: left"></div><br>
		<div style="text-align: center">
			<input id="okbutton" type="button" value="Berechnen"></input>
			<input id="cancelbutton"  type="button" value="Nicht Berechnen"></input>
		</div>
	</div>
	
	Dazu gehören noch die Objekt Variablen der Div's und Button's. Definiert in Globale-This.js.
	
	**********************************************************************************************************
	The module contains functions and event callbacks that are used for
	the use of its own modal dialog window is necessary.
	It simulates the alert () command.
	
	Functions are available, such as:
	----------------------------------------------
	
	resetVars()						-	Resetet die verwendeten Variablen in Globale-this.js
	stopBackgroundWorker()	-	Stops the background worker defined in Background_worker.js
	startBackgroundWorker()	-	Starts the Background Worker, which is in Background_worker.js
												is defined
	goAndWork()					-	Starts the processing (startBackgroundWorker ()) and sends data to the
												Background Worker and to the output window
	Callbach Function resize	-	For the position of the dialog in the middle of the browser
	Callbach Function click		-	For the buttons “OK” and “Cancel”
	Callbach Function keydown-	For the ESC key to cancel the editing and dialog box
												to close
	Callbach Function focus		-	For the focus on the dialog window
	showDialog()					-	Shows or hides the dialog box
	
	
	The following HTML elements are required:
	----------------------------------------------------------
	
	<!-- Bereich <div> zwischen Body und Dialog mit der Abdunkelung -->
	<div id="trans" style="visibility: hidden; position:absolute; width: 0px; height:0px; left: 0px; top: 0px; z-index: 1;"></div>
	<!-- Dialog Box mit Textfeld und Buttons -->
	<div tabindex="-1" style="position: absolute; top: 200px; left:200px; width:300px; height:200px;" role="dialog" aria-lablledby="mydialog" id="dialog" class="box-hidden">
		<div id="txt" role="dialog" aria-lablledby="mydialog" style="color: black; text-align: left"></div><br>
		<div style="text-align: center">
			<input id="okbutton" type="button" value="Berechnen"></input>
			<input id="cancelbutton"  type="button" value="Nicht Berechnen"></input>
		</div>
	</div>
	
	These include the object variables of the Div’s and Button’s. Defined in Global-This.js.
	
*********************************************************************************************************** */

// ******************** Event Listener und Handler **********************
//
/*
	Der Ereignis Callback für das globale Ändern der Fenstergröße
	des Internet Browsers. Event "resize".
	Die Position des Dialogfensters wird dann sogleich auf
	die Größe des Body Bereichs angepasst.
	Die Größe des abgedunkelten Hintergrundes wird ebenfalls
	mit geändert auf auf den Body Bereich angepasst.
	
	The event handler for global window resizing
	of the Internet browser. Event “resize.”
	The position of the dialog window is then immediately set to
	adjusted to the size of the body area.
	The size of the darkened background is also
	with modified adapted to the body area.
*/
window.addEventListener("resize", function(){
	// Breite und Höhe des Bodies ...
	// Width and height of the body ...
	var w = window.innerWidth;
	var h = window.innerHeight;
	// ... und Breite und Höhe des Dialogfensters ...
	// ... and Width and Height of the dialog box ...
	var dlgw = parseInt(dlg.style.width);
	var dlgh = parseInt(dlg.style.height);
	// Die Position der oberen linken Ecke des Dialogfensters
	// neu berechnen ... Wird von der Mitte aus - Hälfte der Body Breite
	// und Hälfte der Body Höhe - berechnet. Hier nur der sichtbare Teil.
	// Später berechnen wir die ganze Höhe des Bodies, den die brauchen
	// wir, um dien Graubereich bei Dialogfenster offen, abzudecken.
	//
	// The position of the upper left corner of the dialog window
	// Recalculate ... Is taken from the middle – half of the body width
	// and half of body height – calculated. Here’s just the visible part.
	// Later we calculate the whole height of the body they need
	// we, to cover the grey area with dialog windows open.
	dlg.style.left = (parseInt(w / 2) - parseInt(dlgw / 2)) + "px";
	dlg.style.top = (parseInt(h / 2) - parseInt(dlgh / 2)) + "px";
	// ... und die Breite und Höhe des abgeblendeten Hintergrunds
	//auf 100% setzen ...
	// ... and the width and height of the faded background
	//set to 100% ...
	objTrans.style.width = 100 + "%";
	objTrans.style.height = 100 + "%";
});
					

// Ereignis Callback des Button "OK" im Dialogfenster ...
// Event listener of the button "OK" into the dialog window
okbutton.addEventListener("click", function(){
	// Dialog Fenster ausblenden und Hintergrund entfernen ...
	// Dialog Hide window and remove background ...
	showDialog("hide", "", 0);
	// ... Variable, die die Verarbeitung anzeigt auf Wahr setzen ...
	// ... Variable that displays the processing set to False ...
	goandwork = true;
});
	
	
// Ereignis Callback des Button "Abbrechen" im Dialogfenster
// Event listener of the button "Cancel" into the dialog window
cancelbutton.addEventListener("click", function(){
	// Dialog Fenster ausblenden und Hintergrund entfernen ...
	// Dialog Hide window and remove background ...
	showDialog("hide", "", 0);
	// ... Variable, die angibt, ob berechnet werden soll auf Falsch setzen =
	// nicht berechnen ...
	// ... Variable that indicates whether to calculate Set to False =
	// do not calculate ...
	goandwork = false;
	// Hält den Background Worker an.
	// Stop the Background Worker.
	stopBackgroundWorker();
	// "Stop" Button gedrückt und Verarbeitung stoppen ...
	// "Stop" Button nicht mehr bedienbar
	// “Stop” button pressed and stop processing ...
	// “Stop” button no longer operable
	objStopCalc.disabled = true;
	// ... und "Berechnen" Button bedienbar machen ...
	// ... and make “Calculate” button operable ...
	objCalc.disabled = false;
	// ... danach Reseten aller Variablen.
	// ... then reset all variables.
	resetVars();
});

		
// Ereignis Callback des Tastendrucks <ESC> oder <RETURN> und Anzeigen des Ergebnisses
// Event callback of the <ESC> or <RETURN> keystroke and display of the result
document.addEventListener("keydown", function(event) {
	// Wenn die Variable für den offenen Dialog gesetzt ist und Escape gedrückt wird ...
	// If the variable for the open dialog is set and Escape is pressed ...
	if (dialogOpen && event.keyCode == 27 ) {
		// Dialog Fenster ausblenden und Hintergrund entfernen  ...
		// dialog Hide window and remove background ...
		showDialog('hide', "",0 );
		// ... Variable, die die Verarbeitung anzeigt auf Falsch setzen ...
		// ... Variable that displays the processing set to False ...
		goandwork = false;
		// Zeigt an ob das Dialogfenster offen ist ...
		// Shows whether the dialog is open ...
		dialogOpen = false;
		// Hält den Background Worker an.
		// Stop the Background Worker.
		stopBackgroundWorker();
		// ... "Stop" Button gedrückt und Verarbeitung stoppen ...
		// "Stop" Button nicht mehr bedienbar
		// ... “Stop” button pressed and stop processing ...
		// “Stop” button no longer operable
		objStopCalc.disabled = true;
		// ... und "Berechnen" Button bedienbar machen ...
		// ... and make “Calculate” button operable ...
		objCalc.disabled = false;
		// Reseten aller Variablen ...
		// Reset all variables ...
		resetVars();

	}
}, true);
		
		
// Ereignis Funktion für den Fokus und den Stop der weiteren Seitenverarbeitung
// Event Callback for the focus and stop of further page processing
document.addEventListener("focus", function(event) {
	// Wenn die Variable für den offenen Dialog gesetzt ist ...
	if (dialogOpen && !dlg.contains(event.target)) {
		// ... Stop der Seitenverarbeitung ...
		event.stopPropagation();
		// ... und Fokus auf den Dialog
		dlg.focus();
	}
}, true);

					
// ******************** Funktionen **********************
					
/*	***************************************************************
	Die Funktion zeigt das Dialogfenster oder verbirgt es.
	Ebenfalls wird die Message ausgegeben.
	Der Dialog wird nicht in Breite und Höhe an die Message-
	länge angepasst. Dies muss von Hand gemacht werden.
					
	Übergabeparameter:
	---------------------------
						
		what = "show" oder "hide"
		msg = "Das ist der Messagetext<br>
				  auch mit neuer Zeile"
	Returnwert:
	----------------

		Keiner
--------------------------------------------------------------------------------
	The function shows or hides the dialog.
	The message is also displayed.
	The dialogue is not extended in width and height to the message-
	length adjusted. This has to be done manually.
					
	Transfer parameter:
	---------------------------
						
		what = "show" or "hide"
		msg = "This is the message text<br>
				  also with new line"
	Return Value:
	----------------

		None
*************************************************************** */
function showDialog(what, msg, jdNM){
	// Breite und Höhe des Bodys ...
	// Width and height of the body ...
	var w = window.innerWidth;
	var h = window.innerHeight;
	// Breite und Höhe des Dialogfensters ...
	// Width and height of the dialog window ...
	var dlgw = parseInt(dlg.style.width);
	var dlgh = parseInt(dlg.style.height);
	// ... zeigt den Dialog an ...
	// ... displays the dialog ...
	if ( what.toLowerCase() == "show"){
		// Wenn der Dialog angezeigt werden soll, dann sichere das Julianische Datum des
		// Neumonds
		// If you want the dialog to be displayed, save the Julian date of the
		// New moons
		jdNewMn =  Number(jdNM);
		// ... und setze die Anzeige Variable auf Wahr ...
		// ... and set the display variable to True ...
		dialogOpen = true;
		// ... der Anzeigestil ist "Block" ...
		// ... the display style is “Block” ...
		dlg.style.display = "block";
		// ... mit dem Fokus auf dem OK-Button des Dialogs ...
		// ... with the focus on the OK button of the dialog ...
		okbutton.focus();
		// ... und der Ausgabe des Messagtextes ...
		// ... and the output of the message text ...
		objTxt.innerHTML = msg;
		// ... Zeichne das Dialogfenster an der oberen linken Ecke von links und oben ...
		// ... Draw the dialog window at the upper left corner from left and top ...
		dlg.style.left = (parseInt(w / 2) - parseInt(dlgw / 2)) + "px";
		dlg.style.top = (parseInt(h / 2) - parseInt(dlgh / 2)) + "px";
		// ... und darunter liegend die abgeblendete Seite ...
		// ... and underneath the faded side ...
		objTrans.style.width = 100 + "%";
		objTrans.style.height = (h + objB.scrollHeight) + "px";
		// ... mit schwarzem Hintergrund ...
		// ... with black background ...
		objTrans.style.backgroundColor = "#000000";
		// ... und der Transparenz 60% ...
		// ... and the transparency 60% ...
		objTrans.style.opacity = 0.6;
		// ... und dann Anzeigen ...
		// ... and then Show ...
		objTrans.style.visibility = "visible";
	// ... ab hier der "hide" Modus ...
	// ... from here the “hide” mode ...
	} else {
		// ... indem die Anzeigevariable auf Falsch gesetz wird ...
		// ... by setting the display variable to False ...
		dialogOpen = false;
		// ... der Display Stil nein ist ...
		// ... the display style no is ...
		dialog.style.display = 'none';
		// ... und das Dialogfenster unsichtbar wird ...
		// ... and the dialog window becomes invisible ...
		objTrans.style.visibility = "hidden";
	}
	// ... und dann zur Funktion, die Entscheidet ob gerechnet wird oder nicht.
	// ... and then to the function, which decides whether to count or not.
	goAndWork();
}// End of showDialog(what)


// Initialisiert den Backgroundworker und startet die Berechnung
// Initializes the background worker and starts the calculation
function goAndWork(){
	// Variable zeigt an, ob eine Berechnung der Sonnenfinsternis gewünscht wird
	// oder nicht
	// Variable indicates whether a calculation of the solar eclipse is desired
	// or not
	if ( goandwork == true){
		// Sende den Start an das grafische Ausgabefenster ...
		// Send the start to the graphical output window ...
		objChannel.postMessage("start=go");
		// Startet den Hintergrund Thread um die Finsternis Daten zu berechnen ... sie
		// werden an den Vordergrund Thread gesendet und dort ausgegeben ...
		// ... aber nur, wenn der Backgroundworker noch nicht definiert und gestartet ist ...
		// Starts the background thread to calculate the eclipse data ... it
		// are sent to the foreground thread and output there ...
		// ... but only if the backgroundworker is not yet defined and started ...
		if(typeof(w)=="undefined") 
			startBackgroundWorker();
			// ... und sende die Programmparameter an den Background Worker...
			// ... and send the program parameters to the Background Worker...
			if (w){
				var val = "";
				// ... und die eingestellte Längengrad Schrittweite an Worker senden, aber nur im
				// Bereich bis 40° ...
				// ... and send the set longitude step width to worker, but only in
				// Range up to 40° ...
				if (objLongStep.value != "" && (objLongStep.value > 0.0 && objLongStep.value <= 40.0) ){
					val += "longstep=" + objLongStep.value + "~";
				// ... ansonsten den Standard von 5° ...
				// ... otherwise the standard of 5° ...
				}else{
					objLongStep.value = "5.0";
					val += "longstep=" + objLongStep.value + "~";
				}
				// ... und die eingestellte Breitengrad Schrittweite, im Bereich bis 20°  an Worker senden ...
				// ... and the set latitude step width, in the range up to 20° send to worker ...
				if (objLatStep.value != "" && ( objLatStep.value > 0.0 && objLatStep.value <= 20.0) ){
					val += "latstep=" + objLatStep.value;
				// ... ansonsten den Standard von 2,5° ...
				// ... otherwise the standard of 2,5° ..
				}else{
					objLatStep.value = "2.5";
					val += "latstep=" + objLatStep.value ;
				}
				// ... die Verarbeitung erfolgt alle 10 ms bei Start, Daten sollen gesendet werden und
				// die Längen- und Breitengrad Schritte an der Worker ...
				// ... the processing is done every 10 ms at startup, data should be sent and
				// the longitude and latitude steps on the worker ...
				w.postMessage("work=getdata~delay=10~" + val);
				// ... Ausgabe der Zeit in ms für eine komplette Berechnung im Hauptprogramm Textfeld...
				// ... Output of the time in ms for a complete calculation in the main program Text field...
				objCalcTime.value = "10 ms";
				// ... und hier Julianisches Datum des Neumonds und Eventbeschreibung 
				//  und einiges mehr an den Backgroundworker ...
				// ... and here Julian date of the new moon and event description 
				// and a lot more to the backgroundworker ...
				w.postMessage("jdNewMoon=" + jdNewMn);
				w.postMessage("occur=" + occur);
				w.postMessage("newmoonday=" + arrDate[0]);
				w.postMessage("newmoonmonth=" + arrDate[1]);
				w.postMessage("newmoonyear=" + arrDate[2]);
				w.postMessage("utfirstcontact=" + retArray[1]);
				w.postMessage("utmaxcontact=" + retArray[2]);
				w.postMessage("utlastcontact=" + retArray[3]);	
			}
							
		}
		// Wenn der Dialog offen ist aber noch keine Option gewählt ist, führe
		// die Funktion Rekursiv aus ... es wird also auf die Entscheidung des Benutzers
		// gewartet, ob berechnet werden soll oder aber abgebrochen werden soll.
		// If the dialog is open but no option is selected, run
		// the function Recursive from ... so it is based on the user’s decision
		// waits to determine whether to calculate or to cancel.
		if ( dialogOpen == true) {
			t = setTimeout(goAndWork, 500);
		}

}// End of goAndWork()
						


// Hält den Background Worker an.
// Stop the Background Worker.
function stopBackgroundWorker(){
	// ... aber nur, wenn er auch läuft ...
	// ... but only if it is running ...
	if(w){
		w.terminate();
		w = undefined;
	}
	
}// End of stopBackgroundWorker()


// Erstellt den neuen Worker und startet die Threads.
// Creates the new worker and starts the threads.
function startBackgroundWorker(){
	var splitData = new Array();
	var splitData0 = new Array();
	var count = 0;
	var mg = 0.0;
	var lo = 0.0;
	var la = 0.0;
						
	// WebWorker erstellen, wenn er nicht definiert ist ...
	// Create WebWorker if it is not defined ...
	if(typeof(Worker)!=="undefined") {
						
		if(typeof(w)=="undefined") {
								
			// Neues WebWorker Objekt generiert aus der JavaScript Datei.
			// Achtung: Bei lokalen Dateien muss der Browser mit dem
			// Kommandline Parameter: "allow-file-access-from-files" gestartet
			// werden. Andernfalls unter einem Webserver starten.
			//
			// New WebWorker object generated from the JavaScript file.
			// Attention: For local files, the browser must be connected with the
			// Commandline parameter: “allow-file-access-from-files” started
			//. Otherwise start under a web server.
			//
			w = new Worker("background_worker.js" );
			
		// Background Worker läuft ...
		}else{
			// ... die Verarbeitung der Daten anfordern, sowie die Verarbeitungszeit einstellen ...
			// ... request the processing of the data and set the processing time ...
			if (w) {
				w.postMessage("work=getdata~delay=" + 10);
				// ... Ausgabe der Zeit in ms für eine komplette Berechnung im Hauptprogramm Textfeld...
				// ... Output of the time in ms for a complete calculation in the main program Text field...
				objCalcTime.value = "10 ms";
			}
			
		}		
		// Ereignis Warteschlange über die alle Daten vom WebWorker eintreffen.
		// Event Que where all the data from the WebWorker arrives.
		w.onmessage = function (event) {
			// Ermittelt die Anfangszeit der Verarbeitung in Millisekunden für die Berechnung der
			// Verarbeitungszeit eines Durchlaufes ...
			// Determines the start time of processing in milliseconds for calculating the
			// Processing time of a run ...
			startProgressTime = Date.now();
			// Empfängt vom Background Worker die Daten und splitet die key=value Wertepaare,
			// die durch eine "~" getrennt sind auf speichert sie im Array workData ...
			//  Receives the data from the background worker and splits the key = value pairs,
			//  which are separated by a “~” on stores them in the array workData ...
			workData = event.data.split("~");
			// ... danach durchlaufe das Array workData mit den key=value Paaren ...
			// ... then run through the array workData with the key=value pairs ...
			for(idx in workData){
				// ... und splitte diese anhand des Trenners "=" auf und lege sie im Array
				// splitData ab. Im Index 0 = key und Index 1 = Data ...
				// ...  and split them using the separator “=” and put them in the array
				// splitData ab.  In index 0 = key and index  1 = Data ...
				splitData = workData[idx].split("=");
				// ... schaue nun nach den Werten der Key's ...
				// ... hier das Julianische Datum des Neumondes und der Finsternis ...
				// ... look now for the values of the key’s ...
				// ...  here the Julian date of the new moon and the darkness ...
				//
				// ... der Längengrad ...
				// ... the longitude ...
				if (splitData[0] == "longitude"){
					lo = splitData[1];
					objLambda.value = lo;
					count++;
				// ... die Bedeckung ...
				// ... the magnitude ...
				}else if (splitData[0] == "magnitude"){
					mg = splitData[1];
					if (mg == -99)
						objMagnitude.value = "0.00";
					else
						objMagnitude.value = mg;
					count++;
				// ... der Breitengrad ...
				// ... the latitude ...
				}else if (splitData[0] == "latitude"){
					la = splitData[1];
					objBeta.value = la;
					count++;
				// ... der Zähler der Berechnungen ist eingetroffen und Berechnungsende ...
				// ... the numerator of the calculations has arrived and calculation end ...
				}else if (splitData[0] == "maincount"){
					// ... hält den Background Worker an...
					// ... stops the Background Worker...
					stopBackgroundWorker();
					// ... Berechnen- und Stop Button ermöglichen/verbieten ...
					// ... Calculate and stop button enable/disable ...
					objCalc.disabled = false;
					objStopCalc.disabled = true;
					s += "\nGesamtberechnungen: " + splitData[1] + "\n";
					// ... und alles in die Textarea als Ergebnis schreiben ...
					// ... and write everything in the text area as a result ...
					if (s != "") s += "\n_________________________________________________________________\n\n";
					objText.value = s;
					// ... die Textarea an das unterste Ende ...
					// ... the text area at the bottom ...
					objText.scrollTop = objText.scrollHeight;
					// Und zum Schluss kopiere noch alles in die Zwischenablage.
					//  And finally, copy everything to the clipboard.
					copyText2Clipboard(objText);
					// ... die definierten Variablen reseten ...
					// ... reset the defined variables ...
					resetVars();
				// ... es wurde ein Programmabbruch angefordert ...
				// ... a program abort has been requested ...
				}else if (splitData[0] == "break"){
					// Hält den Background Worker an.
					// Stop the Background Worker.
					stopBackgroundWorker();
					// ... "Stop" Button gedrückt und Verarbeitung stoppen ...
					// "Stop" Button anzeigen
					// ... “Stop” button pressed and stop processing ...
					// Show “Stop” button
					objStopCalc.disabled = true;
					// ... und "Berechnen" Button unbedienbar machen ...
					// ... and make “Calculate” button unusable ...
					objCalc.disabled = false;
					// ... immer die letzte Zeile in der Textarea anzeigen ...
					// ... always show the last line in the text area ...
					objText.scrollTop = objText.scrollHeight;
					// Und zum Schluss kopiere noch alles in die Zwischenablage.
					//  And finally, copy everything to the clipboard.
					copyText2Clipboard(objText);
					// Die definierten Variablen reseten ...
					// ... reset the defined variables ...
					resetVars();
				// ... die verbrauchte Verarbeitungszeit als Wert 0 - 1 wurde gesendet ...
				// ... the used processing time as value 0 – 1 was sent ...
				}else if (splitData[0] == "percentProgress"){
					// ... und wird als Laufbalken ausgegeben ...
					// ... and is output as a scrollbar ...
					if ((splitData[1] * 100.0) <= 100.0){
						objSpeedbar.style.width = (splitData[1] * 100.0) + "%";
						objProgressbar.style.title =  (splitData[1] * 100.0) + "%";
					}
				// ... Ausgabe der Zeit in ms für eine komplette Berechnung ...
				// ... Output of the time in ms for a complete calculation ...
				}else if (splitData[0] == "returndelay"){
					// ... und Ausgabe im Textfeld ...
					// ... and output in text field ...
					objCalcTime.value = splitData[1] + " ms";
				}
				// Wenn alle 3 Daten (Längengrad, Breitengrad und Bedeckung) empfangen wurden und die Magnitude größer 0 ist,
				// dann alles Formatieren und in den Ergebnisstring "s" eintragen ...
				// If all 3 data (longitude, latitude and coverage) have been received and the magnitude is greater than 0,
				// then formatting everything and entering in the result string “s” ...
				if ( count == 3 ){
					// ... bei Magnitude -99 ersetzte mit 0.0
					// ... at Magnitude -99 replaced with 0.0
					if (mg == -99.0) mg = 0.0;
					// ... und dann formatiert in den String s eintragen ...
					// ... and then enter formatted in the string s ...
					s += "Longitude =  " + LPad(Number(lo).toFixed(2), 7, " ") + "  "  + " Latitude = " + LPad(Number(la).toFixed(2), 7, " ") + "  " + " Magnitude = " + Number(mg).toFixed(4) + "\n";
					// ... und in der Textarea ausgeben ...
					// ... and output in the text area ...
					objText.value = s;
					// ... und nach unten zur letzten Zeile scrollen ...
					// ... and scroll down to the last line ...
					objText.scrollTop = objText.scrollHeight;
					// ... sowie alle Variablen auf 0 setzen ...
					// ... and set all variables to 0 ...
					count = 0;
					mg = 0.0;
					lo = 0.0;
					la = 0.0;
				}
				// Ermittelt die Endzeit der Verarbeitung in Millisekunden für die Berechnung der
				// Verarbeitungszeit eines Durchlaufes ...
				// Determines the end time of processing in milliseconds for calculating the
				// Processing time of a run ...
				stopProgressTime = Date.now();
				// ... wenn die neue Verarbeitungszeit (Stopzeit - Anfangszeit) größer ist als die letzte ... 
				// ... if the new processing time (stop time – start time) is greater than the last ...
				if ( parseInt(stopProgressTime - startProgressTime) > diffProgressTime){
					// ... dann speichere die neue Verarbeitungszeit als die Längere Verarbeitungszeit ...
					// ... then save the new processing time as the Longer processing time ...
					diffProgressTime = parseInt(stopProgressTime - startProgressTime);
					// ... und ist die neue Verarbeitungszeit größer ist als die letzte Wartezeit im Hauptprogramm ist ...
					// ... and if the new processing time is greater than the last waiting time in the main program is ...
					if ( diffProgressTime > DELAY){
						// ... dann passe die Wartezeit im Hauptprogramm an, plus eine kleine Sicherheit von 10ms ...
						// ... then adjust the waiting time in the main program, plus a small security of 10ms ...
						DELAY = parseInt(diffProgressTime + 10);
						// ... und sende es an den Background Worker ...
						// ... and send it to the background worker ... ...
						if (w) 
							// ... und auch noch die Verarbeitungszeiten für die Ausgaben im Hauptprogramm
							// und Grafikausgabefenster anpassen, denn 
							// ... wenn die Rechenzeit im Hauptprogramm größer ist als im Grafik Ausgabefenster ...
							// ... and also the processing times for the outputs in the main program
							// and adjust graphic output window, because 
							// ... if the computation time in the main program is longer than in the graphic output window ...
							if (DELAY > DELAY1)
								// ... dann die Zeit des Hauptprogramms fürs Hauptprogramm verwenden ...
							// ... then use the time of the main program for the main program ...
								w.postMessage("delay=" + DELAY);
							// ... ansonsten verwende für das Hauptprogramm die Zeit der Grafik Ausgabefensters ...
							// ... otherwise use the time of the graphic output windows for the main program ...
							else
								w.postMessage("delay=" + (DELAY1 + 10));
					}
				// Wenn die neue Verarbeitungszeit (Stopzeit - Startzeit) kleiner ist als die letzte und größer als die nötige
				// Rechenzeit... 
				// If the new processing time (stop time – start time) is smaller than the last one and larger than the required one
				// Computing time...
				}else if ( parseInt(stopProgressTime - startProgressTime) < diffProgressTime ) {
					// ... dann speichere die neue Verarbeitungszeit als die geringere ...
					// ... then save the new processing time as the shorter ...
					diffProgressTime = parseInt(stopProgressTime - startProgressTime);
					// ... und wenn sie größer 0.0 ms ist ...
					// ... and if it is greater than 0.0 ms ...
					if ( parseInt(diffProgressTime) > 0 ) {
						// ... dann passe die Wartezeit des Hauptprogramms an plus eine kleine Sicherheit von 10 ms ...
						// ... then adjust the waiting time of the main program plus a small security of 10 ms ...
						DELAY = parseInt(diffProgressTime);
						// ... und sende sie an den Background Worker ...
						// ... and send it to the Background Worker ...
						if (w)
							// ... und auch noch die Verarbeitungszeiten für die Ausgaben im Hauptprogramm
							// und Grafikausgabefenster anpassen, denn 
							// ... wenn die Rechenzeit im Hauptprogramm größer ist als im Grafik Ausgabefenster ...
							// ... and also the processing times for the outputs in the main program
							// and adjust graphic output window, because 
							// ... if the computation time in the main program is longer than in the graphic output window ...
							if (DELAY > DELAY1)
								// ... dann die Zeit des Hauptprogramms fürs Hauptprogramm verwenden ...
								// ... then use the time of the main program for the main program ...
								w.postMessage("delay=" + DELAY + 10);
							// ... ansonsten verwende für das Hauptprogramm die Zeit der Grafik Ausgabefensters ...
							// ... otherwise use the time of the graphic output windows for the main program ...
							else
								w.postMessage("delay=" + DELAY1 + 10);
						}
				}		
				// Zum Schluss fordere neue Daten vom Background Worker an ...
				// Finally, request new data from the background worker ...
				if (w){
					w.postMessage("senddata=true");
					// ... und dasselbe auch zum Grafikausgabefenster senden ...
					// ... dies geschieht vom Hauptprogramm aus ...
					// ... and send the same to the graphic output window ...
					// ... this is done from the main program ...
					if (sendDataOutput == true)
						w.postMessage("senddataoutput=true");
				}
			}// End of for (....)
				
			// Der Wert muss auf 0.0 gesetzt werden, damit bei den Tastatur events
			// die Prüfung auf die Teilbarkeit der Längen- und Breitengrade funktioniert.
			// The value must be set to 0.0 so that keyboard events
			// the test for the divisibility of longitude and latitude is working.
			FIRSTCALC = false;
		}// End of onmessage()

	// Der Browser kennt den WebWorker nicht und gibt eine Fehlermeldung aus.
	// The browser does not know the WebWorker and displays an error message.
	} else {
		showDialog("Show", "Entschuldigung, Ihr Browser unterstützt<br>Multitasking mit Webworker nicht ...", 0);
	}
						
}// End of startBackgroundWorker()


// Funktion um alle deklarierten Variablen auf Anfangswerte zu setzen.
// Function  to set all declared variables to initial values.
function resetVars(){
	DELAYCOUNTER = 10;			// Wartezeit für den Zähler
													// Waiting time for the counter
	splitData = new Array();			// Array nimmt die gespliteten Daten vom HTML Programm auf
													// Array takes the split data from the HTML program
	workData = new Array();			// Array nimmt die einzelnen Werte aus splitData auf
													// Array takes the split data from the splitData array
	sendDataOutput = false;			// Zeigt an, das Grafikfenster Daten haben möchte
													// Shows that the graphic window wants to have data
	sendData = false;						// Zeigt an, das Hauptprogramm Daten haben möchte
													// Shows that the main program wants to have data
	magSolar = new Array();			// Ergebnis der solaren Magnitude
													// Result of the solar magnitude
	jdNewMn = 0.0;						// Julianisches Datum des Neumondes
													// Julian date of the new moon
	occur = "";								// Ereignis des Neumondes von sunMoonEventsOccur()
													// New Moon Event of sunMoonEventsOccur ()
	lo = -180.0;								// Der zu berechnende Längengrad
													// The degree of longitude to be calculated
	la = -90.0;									// Der zu berechnende Breitengrad
													// The degree of latitude to be calculated
	LASTEP = 0.0;							// Schrittweite der Berechnung des Breitengrades
													// Step range of the calculation of the latitude
	LOSTEP = 0.0;							// Schrittweite der Berechnung des Längengrades
													// Step range of the calculation of the longitude
	countDegrees = 0;					// Zähler für die Eingabe der Schrittweite Long + Lat
													// Counter for entering the step length Long + Lat
	mainCount = 1;							// Anzahl der Berechnungen
													// Number of calculations
	calcEnd = false;						// Abbruch des Background-Workers
													// Aborting the background worker
	countProgressWhole = 0.0;		// Anzahl der Berechnungen gesamt
													// Number of calculations total
	percentProgressWhole = 0.0;	// Prozent der schon verarbeiteten Berechnungen
													// Percentage of calculations already processed
	newMoonDay = 0;					// Neumond Tag
													// New moon day
	newMoonMonth = 0;					// Neumond Monat
													// New moon month
	newMoonYear = 0;					// Neumond Jahr
													// New moon year
	sunRize = new Array();				// Ergebnis des Sonnenaufgangs
													// Result of sunrise
	sunSet = new Array();				// Ergebnis des Sonnenuntergangs
													// Result of sunset
	moonRize = new Array();			// Ergebnis des Mondaufgangs
													// Result of moonrise
	moonSet = new Array();			// Ergebnis des Monduntergangs
													// Result of sunset
	eRS = "";									// Returnstring eines Fehlers beim Sonnenauf- und Untergang
													// Return string of an error at sunrise and sunset
	eMoonRS = "";							// Returnstring eines Fehlers beim Mondauf- und Untergang
													// Return string of an error at moonrise and moonset
	sendStrMain = "";						// Sendestring der Daten an das Hauptprogramm
													// Send string of the data to the main program
	sendStrOutput = "";					// Sendestring der Daten an das Grafikfenster
													// Send string of the data to the graphic window
	sendStrMoonOutput = "";			// Sendestring der Monddaten an das Grafikfenster
													// Send string of the moon data to the graphic window
	UTFIRSTCONTACT = 0.0;		// Universal Time des ersten Kontakts Sonne - Mond
													// Universal Time of first contact Sun – Moon
	UTMAXCONTACT = 0.0;			// Universal Time des mittleren Kontakts Sonne - Mond
													// Universal Time of middle contact Sun – Moon
	UTLASTCONTACT = 0.0;			// Universal Time des letzten Kontakts Sonne - Mond
													// Universal Time of last contact Sun – Moon
	dialogOpen = undefined;			// Zeigt an ob das Dialogfenster offen ist
													// Shows whether the dialog is open
	goandwork = undefined;			// Zeigt an, ob der Backgroundworker nach Dialog gestartet werden kann
}// End of resetVars()
				
// ********************************************************	